<?php

namespace IMIncomeLab\WordPress\Core\v2_0;
use \Exception;
use \DateTime;

class License{
    protected $product = null;
    protected $secret = null;
    protected $name = null;
    protected $per_site_licensing = false;
    protected $quiet = false;

    protected $config = null;

    protected $host = 'https://licensing.imincomelab.com';

    private $default_config = array(
        'error' => false,
        'license' => '',
        'expire' => '0000-00-00',
        'verify' => '',
        'features' => array()
    );

    public function __construct($product,$secret,$name,$per_site_licensing = false, $quiet = false){
        $this->product = $product;
        $this->secret = $secret;
        $this->name = $name;
        $this->per_site_licensing = $per_site_licensing;
        $this->quiet = $quiet;
        add_action( 'admin_notices', array($this,'admin_notices') );
        if ( ! wp_next_scheduled( 'imincomelab_license_check' ) ) {
            wp_schedule_event( time(), 'daily', 'imincomelab_license_check' );
        }
        add_action( 'imincomelab_license_check', array($this, 'update'));
        $this->read_config();

        if(defined('METATICK_LICENSING_V2_HOST')){
            $this->host = METATICK_LICENSING_V2_HOST;
        }

        add_action('admin_init', array($this, 'admin_init'));
    }

    public function admin_init(){
        add_action('wp_ajax_imincomelab_license_activate_' . $this->product, array($this, 'wp_ajax_activate'));
        add_action('wp_ajax_imincomelab_license_deactivate_' . $this->product, array($this, 'wp_ajax_deactivate'));
    }

    public function admin_notices(){
        if( $this->is_licensed() || $this->quiet){
            return;
        }
        $message = "{$this->name} is unlicensed";

        if( $this->has_error() )
            $message .= ': ' . $this->get_error();

        echo '<div class="error">';
        echo "<p>$message</p>";
        echo '</div>';
    }

    public function read_config(){
        if (is_multisite() && !$this->per_site_licensing) {
            $config = get_site_option($this->product . '_licensing', array());
        }else {
            $config = get_option($this->product . '_licensing', array());
        }

        $this->config = array_merge($this->default_config, $config);
    }

    public function write_config(){
        if (is_multisite() && !$this->per_site_licensing)
            update_site_option($this->product . '_licensing', $this->config);
        else
            update_option($this->product . '_licensing', $this->config, true);
    }

    public function set_license($license){
        $this->activate($license);
    }

    public function activate($license){
        $this->config['license'] = $license;
        $this->write_config();
        $this->update();
    }

    function get_license(){
        return $this->config['license'];
    }

    function get_expire(){
        return $this->config['expire'];
    }

    function get_verify(){
        return $this->config['verify'];
    }

    function get_error(){
        $license = $this->get_license();
        if( ! $license || $license == ''){
            return "No License Specified";
        }
        return $this->config['error'];
    }

    function has_error(){
        $error = $this->get_error();

        if( $error != null )
            return ! empty($error);

        return false;
    }

    function get_features(){
        return $this->config['features'];
    }

    function get_uid(){
        if( $this->per_site_licensing ) {
            $data = AUTH_KEY . get_current_blog_id();
        }else{
            $data = AUTH_KEY;
        }
        $hash = strtolower(md5(strtolower($data)));
        return $hash;
    }

    function get_device(){
        return get_site_url();
    }

    function is_licensed(){
        $data = $this->get_uid() . $this->get_license() . $this->secret . $this->get_expire();
        $hash = strtolower(md5(strtolower($data)));
        return $hash === $this->get_verify();
    }

    function update(){
        try{
            $url = $this->host . "/api/v1/wordpress/activate/";
            $response = wp_remote_post(
                $url,
                array(
                    'method' => 'POST',
                    'timeout' => 45,
                    'redirection' => 5,
                    'blocking' => true,
                    'sslverify' => false,
                    'body' => array(
                        'product' => $this->product,
                        'key' => $this->get_license(),
                        'uid' => $this->get_uid(),
                        'device' => $this->get_device()
                    ),
                )
            );

            if(is_wp_error($response) && !$this->is_licensed()){
                $this->config['error'] = $response->get_error_message();
                $this->write_config();
                return;
            }

            $message = json_decode(wp_remote_retrieve_body($response));
            if(isset($message->result)){
                if($message->result === 'success'){
                    $options = array(
                        'verify',
                        'features',
                        'expire',
                        'updates',
                        'support',
                        'error'
                    );

                    foreach($options as $option){
                        if(property_exists($message, $option)){
                            $this->config[$option] = $message->$option;
                        }
                    }
                    $this->write_config();
                }
            }else {
                throw new Exception("Invalid Response");
            }
        }catch (Exception $e){
            if(!$this->is_licensed()){
                $this->config['error'] = $e->getMessage();
                $this->write_config();
            }
        }
    }

    function has_feature($check){
        try {
            $features = $this->get_features();
            if(is_object($features)){
                if(property_exists($features, $check)){
                    return true;
                }
            }
            if(is_array($features)){
                if(array_key_exists($check, $features)){
                    return true;
                }
                if(in_array($check, $features)){
                    return true;
                }
            }
        }catch (Exception $e){
            return false;
        }
        return false;
    }

    function wp_ajax_activate(){
        if ( !current_user_can('manage_options') ) {
            echo "Not allowed";
        }
        if(!array_key_exists('key', $_POST)){
            echo 'No Key Received!';
        }else{
            $options = array(
                'verify',
                'features',
                'expire',
                'updates',
                'support',
                'error'
            );

            $this->config['license'] = stripslashes($_POST['key']);

            foreach($options as $option){
                if(array_key_exists($option, $_POST)){
                    $this->config[$option] = stripslashes($_POST[$option]);
                }
            }
            $this->write_config();
            $this->update();

            if($this->is_licensed()){
                echo 'Activated';
            }elseif ($this->has_error()){
                echo $this->get_error();
            }else{
                echo 'An Unknown Error Occurred';
            }
        }
        exit;
    }

    function wp_ajax_deactivate(){
        if ( !current_user_can('manage_options') ) {
            echo "Not allowed";
        }
        $this->config['license'] = $this->config['verify'] = '';
        $this->write_config();
        echo 'License Deactivated';
        exit;
    }

    function get_obfuscated_key(){
        $key = $this->get_license();
        $length = strlen($key);
        $start_length = 0;
        $end_length = 0;
        if($length >= 4 && $length < 8){
            $start_length = 1;
            $end_length = 1;
        }elseif ($length >= 8){
            $start_length = 2;
            $end_length = 2;
        }
        $start = substr($key, 0, $start_length);
        $end = substr($key, 0-$end_length);

        return $start . str_repeat('*', $length - strlen($start) - strlen($end)) . $end;

    }

    function get_default_status() {
        if ($this->is_licensed()) {
            return 'Activated (' . $this->get_obfuscated_key() . ')';
        }
        return 'Not Licensed';
    }

    function has_support(){
        if(!array_key_exists('support', $this->config)){
            return true;
        }
        $support = $this->config['support'];
        return $support >= time();
    }

    function has_updates(){
        if(!array_key_exists('updates', $this->config)){
            return true;
        }
        $updates = $this->config['updates'];
        return $updates >= time();
    }

    function get_default_includes(){
        $includes = array();
        if($this->has_support()){
            $includes[] = 'Support';
        }

        if($this->has_updates()){
            $includes[] = 'Updates';
        }

        if(count($includes) == 0){
            return '';
        }

        return 'Includes ' . implode(' and ', $includes);
    }

    function get_default_actions(){
        if ($this->is_licensed()) {
            $id = 'imincomelab_license_deactivate_' . $this->product;
            return '<button class="button" id="' . $id . '" data-product="' . $this->product . '">Deactivate License</button>';
        }else{
            $id = 'imincomelab_license_activate_' . $this->product;
            return '<button class="button" id="' . $id . '" data-product="' . $this->product . '">Activate License</button>';
        }
    }

    function get_javascript_activation_url(){
        $parameters = array(
            'product' => $this->product,
            'uid' => $this->get_uid(),
            'device' => $this->get_device()
        );

        $url = $this->host . "/api/v1/wordpress/jsactivate/?" . http_build_query($parameters);
        return $url;
    }

    function widget($return=false){
        if($this->is_licensed()) {
            $status = apply_filters('imincomelab_license_status_' . $this->product, $this->get_default_status(), $this);
            $description = apply_filters('imincomelab_license_description_' . $this->product, '', $this);
            $includes = apply_filters(
                'imincomelab_license_includes_' . $this->product, $this->get_default_includes(), $this
            );
            $actions = apply_filters(
                'imincomelab_license_actions_' . $this->product, $this->get_default_actions(), $this
            );
            $rows = array(
                'status' => $status,
                'description' => $description,
                'includes' => $includes,
                'actions' => $actions,
            );
        }else{
            $status = apply_filters('imincomelab_license_status_' . $this->product, $this->get_default_status(), $this);

            $actions = apply_filters(
                'imincomelab_license_actions_' . $this->product, $this->get_default_actions(), $this
            );
            $rows = array(
                'status' => $status,
                'actions' => $actions,
            );
        }
        if($return){
            ob_start();
        }
        ?>
        <div class="imincomelab-license-widget imincomelab-license-widget-<?php echo $this->product; ?>">
            <?php foreach($rows as $name => $content): ?>
                <?php if($content and strlen($content) > 0): ?>
                    <div class="imincomelab-license-<?php echo $name; ?>"><?php echo $content; ?></div>
                <?php endif; ?>
            <?php endforeach; ?>
        </div>
        <style>
            .imincomelab-license-widget {
                position: relative;
                padding: 20px;
                background-color: white;
                border: 1px solid #d2d2d2;
                margin: 1px;
                max-width: 25em;
                box-sizing: border-box;
            }
            .imincomelab-license-status,
            .imincomelab-license-description,
            .imincomelab-license-includes {
                margin-bottom:5px;
            }
            .imincomelab-license-description {
                font-weight: bold;
            }
            .imincomelab-license-includes {
                opacity: 0.8;
            }
            .imincomelab-license-actions {
                text-align: right;
            }
        </style>
        <script>
            (function($){
                var ajax_url = <?php echo json_encode(admin_url( 'admin-ajax.php' )); ?>;

                $('#imincomelab_license_activate_<?php echo $this->product; ?>').click(function(e){
                    <?php if(array_key_exists('allow_key_entry', $_GET)): ?>
                        var key = prompt('Enter your <?php echo $this->name; ?> key:');
                        if(key){
                            $.post(
                                ajax_url, {
                                    'key': key,
                                    'action': 'imincomelab_license_activate_<?php echo $this->product; ?>'
                                },
                                function(response) {
                                    location.reload();
                                }
                            );
                        }
                    <?php else: ?>
                        var url = <?php echo json_encode($this->get_javascript_activation_url()); ?>;
                        var specs = 'height=400,width=400,left=50,location=no,menubar=no,top=50';
                        var licensing_window = window.open(url, 'license', specs);
                        window.addEventListener(
                            "message",
                            function(message){
                                try {
                                    if(message.data
                                        && message.data.action
                                        && message.data.action === 'activate'
                                        && message.data.product
                                        && message.data.product === '<?php echo $this->product; ?>'){

                                        $.post(
                                            ajax_url, {
                                                'key': message.data.key,
                                                'verify': message.data.verify,
                                                'features': message.data.features,
                                                'expire': message.data.expire,
                                                'updates': message.data.updates,
                                                'support': message.data.support,
                                                'error': message.data.error,
                                                'action': 'imincomelab_license_activate_<?php echo $this->product; ?>'
                                            },
                                            function(response) {
                                                licensing_window.close();
                                                location.reload();
                                            }
                                        );
                                    }
                                }catch(e){
                                    console.log('Unhandled error: ' + e);
                                }
                            },
                            false
                        );

                    <?php endif; ?>
                    e.preventDefault();
                });
                $('#imincomelab_license_deactivate_<?php echo $this->product; ?>').click(function(e){
                    if(confirm("Are you sure you wish to deactivate <?php echo $this->name; ?>?")) {
                        $.post(
                            ajax_url, {
                                'action': 'imincomelab_license_deactivate_<?php echo $this->product; ?>'
                            },
                            function(response) {
                                alert(response);
                                location.reload();
                            }
                        );
                    }
                    e.preventDefault();
                });
            })(jQuery);
        </script>
        <?php
        if($return){
            return ob_get_clean();
        }
    }
}
